@@ -30,7 +30,6 @@ from pynicotine.core import core
 from pynicotine.gtkgui.application import GTK_API_VERSION
 from pynicotine.gtkgui.widgets import clipboard
 from pynicotine.gtkgui.widgets.accelerator import Accelerator
-from pynicotine.gtkgui.widgets.dialogs import EntryDialog
 from pynicotine.utils import TRANSLATE_PUNCTUATION
 
 
@@ -351,8 +350,11 @@ class UserPopupMenu(PopupMenu):
 
         self.username = username
         self.tab_name = tab_name
+        self.popup_menu_private_rooms = None
 
-        self.setup_user_menu(username)
+        if tab_name != "private_rooms":
+            self.popup_menu_private_rooms = UserPopupMenu(self.application, username=username, tab_name="private_rooms")
+            self.setup_user_menu(username)
 
     def setup_user_menu(self, username):
 
@@ -376,14 +378,15 @@ class UserPopupMenu(PopupMenu):
             self.add_items(("$" + _("_Add Buddy"), self.on_add_to_list))
 
         self.add_items(
-            ("#" + _("_Gift Privileges…"), self.on_give_privileges),
             ("", None),
             ("$" + _("Ban User"), self.on_ban_user),
             ("$" + _("Ignore User"), self.on_ignore_user),
             ("", None),
             ("$" + _("Ban IP Address"), self.on_ban_ip),
             ("$" + _("Ignore IP Address"), self.on_ignore_ip),
-            ("#" + _("Show IP A_ddress"), self.on_show_ip_address)
+            ("#" + _("Show IP A_ddress"), self.on_show_ip_address),
+            ("", None),
+            (">" + _("Private Rooms"), self.popup_menu_private_rooms)
         )
 
     def update_username_item(self):
@@ -408,10 +411,12 @@ class UserPopupMenu(PopupMenu):
         self.username = username
         self.update_username_item()
 
+        if self.popup_menu_private_rooms is not None:
+            self.popup_menu_private_rooms.set_user(self.username)
+
     def toggle_user_items(self):
 
         self.editing = True
-        self.actions[_("_Gift Privileges…")].set_enabled(bool(core.users.privileges_left))
 
         local_username = core.users.login_username or config.sections["server"]["login"]
         add_to_list = _("_Add Buddy")
@@ -429,16 +434,17 @@ class UserPopupMenu(PopupMenu):
             self.actions[action_id].set_enabled(GLib.Variant("b", self.username != local_username or value))
             self.actions[action_id].set_state(GLib.Variant("b", value))
 
+        self.popup_menu_private_rooms.populate_private_rooms()
+        self.popup_menu_private_rooms.update_model()
+
+        private_rooms_enabled = (self.popup_menu_private_rooms.items and self.username != core.users.login_username)
+        self.actions[_("Private Rooms")].set_enabled(private_rooms_enabled)
+
         self.editing = False
 
-    def populate_private_rooms(self, popup):
+    def populate_private_rooms(self):
 
-        popup.clear()
-
-        if self.username is None:
-            return
-
-        popup.set_user(self.username)
+        self.clear()
 
         for room, data in core.chatrooms.private_rooms.items():
             is_owned = core.chatrooms.is_private_room_owned(room)
@@ -448,23 +454,24 @@ class UserPopupMenu(PopupMenu):
                 continue
 
             if self.username in data["users"]:
-                popup.add_items(
-                    ("#" + _("Remove from Private Room %s") % room, popup.on_private_room_remove_user, room))
+                self.add_items(
+                    ("#" + _("Remove from Private Room %s") % room, self.on_private_room_remove_user, room))
             else:
-                popup.add_items(("#" + _("Add to Private Room %s") % room, popup.on_private_room_add_user, room))
+                self.add_items(
+                    ("#" + _("Add to Private Room %s") % room, self.on_private_room_add_user, room))
 
             if not is_owned:
                 continue
 
             if self.username in data["operators"]:
-                popup.add_items(
-                    ("#" + _("Remove as Operator of %s") % room, popup.on_private_room_remove_operator, room))
-            else:
-                popup.add_items(("#" + _("Add as Operator of %s") % room, popup.on_private_room_add_operator, room))
+                self.add_items(
+                    ("#" + _("Remove as Operator of %s") % room, self.on_private_room_remove_operator, room))
 
-            popup.add_items(("", None))
+            elif self.username in data["users"]:
+                self.add_items(
+                    ("#" + _("Add as Operator of %s") % room, self.on_private_room_add_operator, room))
 
-        popup.update_model()
+            self.add_items(("", None))
 
     def update_model(self):
         super().update_model()
@@ -569,40 +576,3 @@ class UserPopupMenu(PopupMenu):
 
     def on_copy_user(self, *_args):
         clipboard.copy_text(self.username)
-
-    def on_give_privileges_response(self, dialog, _response_id, _data):
-
-        days = dialog.get_entry_value()
-
-        if not days:
-            return
-
-        try:
-            days = int(days)
-            core.users.request_give_privileges(self.username, days)
-
-        except ValueError:
-            self.on_give_privileges(error=_("Please enter number of days."))
-
-    def on_give_privileges(self, *_args, error=None):
-
-        core.users.request_check_privileges()
-
-        if core.users.privileges_left is None:
-            days = _("Unknown")
-        else:
-            days = core.users.privileges_left // 60 // 60 // 24
-
-        message = (_("Gift days of your Soulseek privileges to user %(user)s (%(days_left)s):") %
-                   {"user": self.username, "days_left": _("%(days)s days left") % {"days": days}})
-
-        if error:
-            message += "\n\n" + error
-
-        EntryDialog(
-            parent=self.application.window,
-            title=_("Gift Privileges"),
-            message=message,
-            action_button_label=_("_Give Privileges"),
-            callback=self.on_give_privileges_response
-        ).present()
